from youtubesearchpython.videos__search import SearchVideos
from youtubesearchpython.playlist__search import SearchPlaylists

from youtubesearchpython.videos__search import SearchVideos as searchYoutube 


import json

from youtubesearchpython.__requesthandler import RequestHandler
from youtubesearchpython.videos__pagehandler import PageHandler
from youtubesearchpython.videos__scripthandler import ScriptHandler


class SearchVideos(RequestHandler, PageHandler, ScriptHandler):
    '''
    Search for videos in YouTube.
    Parameters
    ----------
    keyword : str
        Used as a query to search for videos in YouTube.
    offset : int, optional
        Offset for result pages on YouTube. Defaults to 1.
    mode : str
        Search result mode. Can be 'json', 'dict' or 'list'.
    maxResults : int, optional
        Maximum number of video results. Defaults to 20.
    Methods
    -------
    search()
        Searches for the videos in YouTube
    result()
        Returns the videos fetched from YouTube by search().
    '''
    networkError = False
    validResponse = False

    def __init__(self, keyword, offset = 1, mode = "json", max_results = 20):
        self.offset = offset
        self.mode = mode
        self.keyword = keyword
        self.max_results = max_results
        self.searchPreferences = "EgIQAQ%3D%3D"
        self.main()

    def main(self):
        self.request()
        if self.networkError:
            self.networkError = True
        else:

            if not self.validResponse:
                self.scriptResponseHandler()
            
            if self.validResponse:
                self.pageResponseHandler()

    def result(self):
        '''
        Returns
        -------
        None, str, dict, list
            Returns video results from YouTube. Returns None, if network error occurs.
        '''
        if self.networkError:
            return None

        else:

            result = []
            
            if self.mode in ["json", "dict"]:

                for index in range(len(self.ids)):
                    result_index = {
                        "index": index,
                        "id": self.ids[index],
                        "link": self.links[index],
                        "title": self.titles[index],
                        "channel": self.channels[index],
                        "duration": self.durations[index],
                        "views": self.views[index],
                        "thumbnails": self.thumbnails[index]
                    }
                    result+=[result_index]

                if self.mode == "json":
                    return json.dumps({"search_result": result}, indent=4)
                else:
                    return {"search_result": result}
                    
            elif self.mode == "list":
                
                for index in range(len(self.ids)):
                    list_index=[
                            index,
                            self.ids[index],
                            self.links[index],
                            self.titles[index],
                            self.channels[index],
                            self.durations[index],
                            self.views[index],
                            self.thumbnails[index]
                    ]
                    result+=[list_index]
                
                return result


class ScriptHandler:
    def scriptResponseHandler(self):     
        self.links = []
        self.ids = []
        self.titles = []

        self.pageSource = self.page.split('":"')

        for index in range(0, len(self.pageSource) - 1, 1):
            
            ''' Setting Playlist ID and link. '''

            if self.pageSource[index][-10:] == 'playlistId':
                if self.pageSource[index+1].split('"')[0] not in self.ids:
                    if self.pageSource[index+1].split('"')[0] == "WL":
                        pass
                    else:
                        self.ids+=[self.pageSource[index+1].split('"')[0]]
                        self.links+=["https://www.youtube.com/playlist?list=" + self.pageSource[index+1].split('"')[0]]

            ''' Setting Playlist Title. '''

            if self.pageSource[index][-20:] == '"title":{"simpleText' and self.pageSource[index+1][-3:] == 'url':
                self.titles+=[self.pageSource[index+1].split('"},"')[0].replace("\\u0026", "&")]

            if len(self.titles) > self.max_results and len(self.ids) > self.max_results:
                max_results = min(len(self.titles), len(self.ids))
                self.titles = self.titles[0:max_results]
                self.ids = self.ids[0:max_results]
                self.links = self.links[0:max_results]
                break
                


import sys

if sys.version_info[0] == 2:
    from urllib import urlencode, urlopen
else:
    from urllib.request import urlopen
    from urllib.parse import urlencode


class RequestHandler:
    def request(self):
        try:
            query = urlencode({
                "search_query": self.keyword,
                "page": self.offset,
                "sp": self.searchPreferences
            })
            request = "https://www.youtube.com/results" + "?" + query
            response = urlopen(request).read()
            self.page = response.decode('utf_8')

            if self.page[0:29] == '  <!DOCTYPE html><html lang="':
                self.validResponse = True

        except: 
            self.networkError = True



class ScriptHandler:
    def scriptResponseHandler(self):
        self.links = []
        self.ids = []
        self.titles = []
        self.channels = []
        self.views = []
        self.durations = []
        self.thumbnails = []

        self.pageSource = self.page.split('":"')

        for index in range(0, len(self.pageSource) - 1, 1):
            
            ''' Setting Video Durations And View Counts. '''

            if self.pageSource[index][-14:] == '}},"simpleText' and self.pageSource[index+1][-28:] == '"viewCountText":{"simpleText':
                durationBuffer = ""
                viewCountBuffer = 0
                for character in self.pageSource[index+1]:
                    if character!= '"':
                        durationBuffer+=character
                    else:
                        break
                for character in self.pageSource[index+2].split()[0]:
                    if character.isnumeric():
                        viewCountBuffer = viewCountBuffer * 10 + int(character)
                self.durations[-1] = durationBuffer
                self.views[-1] = viewCountBuffer

            ''' Setting Video Links, IDs And Thumbnails. '''

            if self.pageSource[index][0:9] == '/watch?v=':
                id = self.pageSource[index][9:20]
                modes = ["default", "hqdefault", "mqdefault", "sddefault", "maxresdefault"]
                self.ids+=[id]
                self.links+=['https://www.youtube.com/watch?v='+ id]
                thumbnailbuffer = []
                for mode in modes:
                    thumbnailbuffer+=["https://img.youtube.com/vi/" + id + "/" + mode + ".jpg"]
                self.thumbnails+=[thumbnailbuffer]
            
            ''' Setting Video Titles. '''

            if self.pageSource[index][-23:] == '"title":{"runs":[{"text' and self.pageSource[index+1][-44:] == '"accessibility":{"accessibilityData":{"label':
                titleBuffer = ""
                for subIndex in range(len(self.pageSource[index+1])):
                    if self.pageSource[index+1][subIndex: subIndex+2] != '}]':
                        ''' For getting rid of " written as \" '''
                        if self.pageSource[index+1][subIndex] == '"' and self.pageSource[index+1][subIndex+1: subIndex+3] != '}]':
                            titleBuffer = titleBuffer[:-1]
                        titleBuffer+=self.pageSource[index+1][subIndex]
                    else:
                        titleBuffer = titleBuffer[:-1]
                        break
                self.titles+=[titleBuffer.replace("\\u0026", "&")]

                self.views+=["LIVE"]
                self.durations+=["LIVE"]
                self.channels+= [""]

            ''' Setting Video Channels. '''

            if self.pageSource[index][-32:] == '"longBylineText":{"runs":[{"text' and self.pageSource[index+1][-42:] == '"navigationEndpoint":{"clickTrackingParams':
                channelBuffer = ""
                for character in self.pageSource[index+1]:
                    if character!= '"':
                        channelBuffer+=character
                    else:
                        break
                try:
                    self.channels[-1] = channelBuffer.replace("\\u0026", "&")
                except:
                    pass

            if len(self.ids) + 1 > self.max_results:
                break